//
//  MainViewController.m
//  LocationApp_Sample
//
//  Copyright (c) 2012 ZENRIN DataCom CO., LTD. All Rights Reserved.
//

#import "MainViewController.h"

#import <AudioToolbox/AudioToolbox.h>
#import "AppSettingViewController.h"
#import "LibSettingViewController.h"
#import "Storyboard.h"
#import "LMaUtility.h"
#import "ServerConnection.h"

/* １送信あたり最大送信履歴数 */
#define kSendLogCountMax 1000

#define kTokenId          @"access_token"
#define kTokenExpireDate  @"expiresDate"

/* AlertDialogの種類 */
enum
{
    AlertDialogType_Default = 0,     // デフォルト
    AlertDialogType_CurrentLocation, // 現在地取得
};

/* ActionSheetの種類 */
enum
{
    ActionSheetType_Default = 0,     // デフォルト
    ActionSheetType_CurrentSettings, // 設定取得
};

static NSString *const EDITABLE = @"editable";

@interface MainViewController () <UIActionSheetDelegate>
{
    __strong DataManager *_dataManager;
    
    NSMutableArray *_locationLogs;
    NSMutableDictionary *_tokenInfo;
}

/* ログを追加 */
-(void)_setLogText:(NSString *)test;
/* ログ用のDateFormatter取得 */
- (NSDateFormatter *)_logDateFormatter:(NSString *)dateFormat;

/* ログファイルのパス取得 */
- (NSString *)_locationLogFilePath;
/* ログファイルを作成する */
- (void)_createLocationLogFile:(NSString *)filePath;
/* ログファイルを保存する */
- (void)_saveLogFile:(NSString *)logString;
/* ログデータの作成 */
- (NSString *)_createLogData:(LocationInfo *)locationInfo;
/* ステータスログデータの作成 */
- (NSString *)_createStatusLogData:(MeasuringType)measureStatus reason:(MeasuringTypeChangeReason)reason;

/* 文字列アラート表示 */
- (void)_showTextAlert:(NSString *)title message:(NSString *)message;

/* 取得した位置を履歴に格納 */
- (void)_stackLocation:(LocationInfo *)locationInfo;
/* トークン取得通信 */
- (void)_requestGetToken:(void (^)(void))successHandler;
/* 位置履歴通信 */
- (void)_requestUploadLogs;
/* 通信結果をコンソールログに出力 */
- (void)_outputConnectionResultToConsole:(NSString *)log response:(NSHTTPURLResponse *)response data:(NSDictionary *)data error:(NSError *)error;
/* 位置蓄積サーバ送信用Dictionary生成 */
- (NSDictionary *)_createlocationLogDictionary:(NSArray *)logs;
/* ログ画面に出力 */
- (void)_setLogTextInUIThread:(NSString *)log;

@end


@implementation MainViewController

- (instancetype)initWithCoder:(NSCoder *)aDecoder
{
    self = [super initWithCoder:aDecoder];
    if (self) {
        _dataManager = [[DataManager alloc] init];
        [_dataManager restoreSettings];
        
        _tokenInfo = [NSMutableDictionary dictionary];
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    /* LocationManagerのデリゲート設定 */
    [LocationManager defaultLocationManager].delegate = self;
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    
    [_logTextView setText:@""];
}

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if ([segue.identifier isEqualToString:kAppSettingViewControllerStoryboardSegueID])
    {
        /* アプリ設定画面の表示 */
        if ([segue.destinationViewController isKindOfClass:[AppSettingViewController class]])
        {
            AppSettingViewController *appSettingViewController = (AppSettingViewController *)segue.destinationViewController;
            appSettingViewController.dataManager = _dataManager;
        }
    }
    else if ([segue.identifier isEqualToString:kLibSettingViewControllerStoryboardSegueID])
    {
        /* ライブラリ設定画面の表示 */
        if ([segue.destinationViewController isKindOfClass:[LibSettingViewController class]])
        {
            LibSettingViewController *libSettingViewController = (LibSettingViewController *)segue.destinationViewController;
            if (([(NSDictionary *)sender objectForKey:EDITABLE] != nil))
            {
                /* 更新 */
                libSettingViewController.isEditable = YES;
                libSettingViewController.dataManager = _dataManager;
            }
            else
            {
                /* 設定値確認 */
                if (sender == nil)
                {
                    libSettingViewController.dataManager = nil;
                }
                else
                {
                    libSettingViewController.dataManager = [[DataManager alloc] initWithLibSettings:sender];
                }
            }
        }
    }
}

#pragma mark IBAction

/* 測位開始ボタン押下 */
- (IBAction)startDown:(id)sender
{
    _startButton.enabled = NO;

    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_BACKGROUND, 0), ^(void) {
        @synchronized (self) {
            NSMutableString *log = [NSMutableString stringWithString:@"測位開始："];

            /* 測位開始 */
            NSError *error = nil;

            MeasuringStartResult measuringResult = [[LocationManager defaultLocationManager] measuringStart:[self->_dataManager libSettings] error:&error];

            switch (measuringResult) {
                case MeasuringStartResult_Standard:
                    [log appendString:@"成功(Standard)"];
                    break;

                case MeasuringStartResult_SignificantChange:
                    [log appendString:@"成功(SignificantChange)"];
                    break;

                case MeasuringStartResult_RegionVisit:
                    [log appendString:@"成功(RegionVisit)"];
                    break;

                case MeasuringStartResult_Error:
                    if (error != nil) {
                        [self _outputConnectionResultToConsole:@"Library Token request failed" response:nil data:nil error:error];
                    }
                default:
                    [log appendString:@"失敗"];
                    break;
            }

            [log appendFormat:@"\n設定 = \n%@\n\n", [[self->_dataManager libSettings] description]];

            dispatch_async(dispatch_get_main_queue(), ^{
                [self _setLogText:log];
                self->_startButton.enabled = YES;
            });
        }
    });
}

/* 測位停止ボタン押下 */
- (IBAction)stopDown:(id)sender
{
    _stopButton.enabled = NO;
    
    @synchronized (self) {
        NSMutableString *log = [NSMutableString stringWithString:@"測位停止\n"];
        
        /* 測位を停止する */
        [[LocationManager defaultLocationManager] measuringStop];
        
        [self _setLogText:log];
    }
    _stopButton.enabled = YES;
}

/* ログ消去ボタン押下 */
- (IBAction)logDeleteDown:(id)sender
{
    _logTextView.text = @"";
    [_logTextView setNeedsDisplay];
}

/* 現在地取得ボタン押下 */
- (IBAction)currentLocationDown:(id)sender
{
    _currentLocationButton.enabled = NO;

    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_BACKGROUND, 0), ^(void) {
        @synchronized (self) {
            double accuracy = 0.0;
            if (self->_dataManager.accuracy != FloatEmptyValue) {
                accuracy = self->_dataManager.accuracy;
            }

            NSError *error = nil;
            LocationInfo *currentLocationInfo = [[LocationManager defaultLocationManager] currentLocation:accuracy
                                                apiKey:self->_dataManager.libApiKey
                                                clientId:self->_dataManager.libTokenClientID
                                                secret:self->_dataManager.libTokenSecret
                                                authEnv:self->_dataManager.libAuthEnv
                                                error:&error];

            /* 出力ログの形成 */
            NSMutableString *log = [NSMutableString string];
            NSString *title;
            if (error == nil) {
                /* 位置情報をメモリに保持 */
                [self _stackLocation:currentLocationInfo];

                /* Local設定 */
                [log appendFormat:@"\n日付: %@\n", currentLocationInfo.date];
                [log appendFormat:@"時間: %@\n", currentLocationInfo.time];
                [log appendFormat:@"緯度: %+f\n", currentLocationInfo.latitude];
                [log appendFormat:@"経度: %+f\n", currentLocationInfo.longitude];
                [log appendFormat:@"水平誤差: %+f\n", currentLocationInfo.horizontalAccuracy];
                [log appendFormat:@"垂直誤差: %+f\n", currentLocationInfo.verticalAccuracy];
                [log appendFormat:@"標高: %+f\n", currentLocationInfo.altitude];
                [log appendFormat:@"速度: %+f\n", currentLocationInfo.speed];
                [log appendFormat:@"方位: %+f\n\n", currentLocationInfo.bearing];
                title = @"現在地取得結果";
            }
            else
            {
                [log appendFormat:@"\nエラーコード: %ld\n", (long)error.code];
                [log appendFormat:@"\n詳細: %@\n", error.localizedDescription];
                title = @"現在地取得エラー";
            }

            dispatch_async(dispatch_get_main_queue(), ^{
                [self _showTextAlert:title message:log];
                self->_currentLocationButton.enabled = YES;
            });
        }
    });
}

/* 設定値取得ボタン押下 */
- (IBAction)currentSettingsDown:(id)sender
{
    NSString *cancelMenuTitle = @"Cancel";
    NSString *setValuesMenuTitle = @"設定値を取得";
    NSString *measureValuesMenuTitle = @"動作値を取得";
    
    if (NSClassFromString(@"UIAlertController") != nil)
    {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:nil
                                                                                 message:nil
                                                                          preferredStyle:UIAlertControllerStyleActionSheet];
        void (^handler)(UIAlertAction *action) = ^(UIAlertAction *action){
            if ([action.title isEqualToString:setValuesMenuTitle])
            {
                /* 設定値を取得 */
                NSDictionary *librarySettings = [[LocationManager defaultLocationManager] librarySettings:NO];
                [self performSegueWithIdentifier:kLibSettingViewControllerStoryboardSegueID sender:librarySettings];
            }
            else if ([action.title isEqualToString:measureValuesMenuTitle])
            {
                /* 動作値を取得 */
                NSDictionary *librarySettings = [[LocationManager defaultLocationManager] librarySettings:YES];
                [self performSegueWithIdentifier:kLibSettingViewControllerStoryboardSegueID sender:librarySettings];
            }
        };
        [alertController addAction:[UIAlertAction actionWithTitle:cancelMenuTitle
                                                            style:UIAlertActionStyleCancel
                                                          handler:handler]];
        [alertController addAction:[UIAlertAction actionWithTitle:setValuesMenuTitle
                                                            style:UIAlertActionStyleDefault
                                                          handler:handler]];
        [alertController addAction:[UIAlertAction actionWithTitle:measureValuesMenuTitle
                                                            style:UIAlertActionStyleDefault
                                                          handler:handler]];
        
        [self presentViewController:alertController animated:YES completion:nil];
    }
}

/* バージョン取得ボタン押下 */
- (IBAction)libraryVersionDown:(id)sender
{
    /* ライブラリのバージョン情報取得 */
    NSString *libraryVersionString = [LocationManager defaultLocationManager].libraryVersion;
    [self _showTextAlert:@"バージョン情報" message:libraryVersionString];
}

/* メニュー押下 */
- (IBAction)menuDown:(id)sender {
    
    NSString *cancelMenuTitle = @"Cancel";
    NSString *libSettingsMenuTitle = @"ライブラリ設定";
    NSString *appSettingsMenuTitle = @"アプリ設定";
    
    if (NSClassFromString(@"UIAlertController") != nil)
    {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:nil
                                                                                 message:nil
                                                                          preferredStyle:UIAlertControllerStyleActionSheet];
        void (^handler)(UIAlertAction *action) = ^(UIAlertAction *action){
            if ([action.title isEqualToString:libSettingsMenuTitle])
            {
                /* ライブラリ設定画面へ遷移 */
                [self performSegueWithIdentifier:kLibSettingViewControllerStoryboardSegueID
                                          sender:[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithBool:YES], EDITABLE, nil]];
            }
            else if ([action.title isEqualToString:appSettingsMenuTitle])
            {
                /* アプリ設定を取得 */
                [self performSegueWithIdentifier:kAppSettingViewControllerStoryboardSegueID
                                          sender:nil];
            }
        };
        [alertController addAction:[UIAlertAction actionWithTitle:cancelMenuTitle
                                                            style:UIAlertActionStyleCancel
                                                          handler:handler]];
        [alertController addAction:[UIAlertAction actionWithTitle:libSettingsMenuTitle
                                                            style:UIAlertActionStyleDefault
                                                          handler:handler]];
        [alertController addAction:[UIAlertAction actionWithTitle:appSettingsMenuTitle
                                                            style:UIAlertActionStyleDefault
                                                          handler:handler]];
        
        [self presentViewController:alertController animated:YES completion:nil];
    }
}

#pragma mark LocationManagerDelegate

/* 測位情報を通知するコールバック関数 */
- (void)measuringResult:(LocationInfo *)locationInfo status:(LocationManagerStatus)status error:(NSError *)error
{
    if (status == LocationManagerStatusError) {
        [self _setLogText:@"測位に失敗しました。\n"];
        if (_dataManager.debugMode == YES ) {
            NSMutableString *log = nil;
            log = [NSMutableString string];
            NSDateFormatter *formatter = [self _logDateFormatter:@"yyyyMMdd HH:mm:ss"];
            [log appendString:[formatter stringFromDate:[NSDate date]]];
            [log appendString:@",測位失敗"];
            [log appendString:@"\n"];
            
            [self _saveLogFile:log];
        }
    } else {
        /* 位置情報をメモリに保持 */
        [self _stackLocation:locationInfo];

        /* 出力ログの形成 */
        NSMutableString *log = [NSMutableString string];

        /* Local設定 */
        NSDateFormatter *formatter = [self _logDateFormatter:@"yyyy-MM-dd HH:mm:ss"];
        [log appendFormat:@"現在: %@\n", [formatter stringFromDate:[NSDate date]]];
        [log appendFormat:@"日付: %@          ", locationInfo.date];
        [log appendFormat:@"時間: %@\n", locationInfo.time];
        [log appendFormat:@"緯度: %+f       ", locationInfo.latitude];
        [log appendFormat:@"経度: %+f\n", locationInfo.longitude];
        [log appendFormat:@"水平誤差: %+f\n", locationInfo.horizontalAccuracy];
        [log appendFormat:@"垂直誤差: %+f\n",locationInfo.verticalAccuracy];
        [log appendFormat:@"標高: %+f        ", locationInfo.altitude];
        [log appendFormat:@"速度: %+f\n", locationInfo.speed];
        [log appendFormat:@"方位: %+f\n\n", locationInfo.bearing];
        
        [self _setLogText:log];

        if (_dataManager.debugMode == YES) {
            NSString *logData = [self _createLogData:locationInfo];
            if (logData != nil) {
                [self _saveLogFile:logData];
            }
        }
    }
}

/* 測位方法変更を通知するコールバック関数 */
- (void)changeMeasureStatus:(MeasuringType)measureStatus reason:(MeasuringTypeChangeReason)reason
{
    NSString *log = [self _createStatusLogData:measureStatus reason:reason];
    
    if ([log length] > 0) {
        [self _setLogText:log];
        if (_dataManager.debugMode == YES) {
            [self _saveLogFile:log];
        }
    }
}

#pragma mark Public

/* デバッグ用ログを追加 */
- (void)setDebugLogText:(NSString *)logString
{
    if (_dataManager.debugMode == YES) {
        if (logString != nil)
        {
            NSDateFormatter *formatter = [self _logDateFormatter:@"yyyyMMdd HH:mm:ss"];
            NSString *dateStr = [formatter stringFromDate:[NSDate date]];
            
            NSString *logData = [NSString stringWithFormat:@"%@,%@\n", dateStr, logString];
            [self _saveLogFile:logData];
        }
    }
}

#pragma mark Private

/* ログを追加 */
-(void)_setLogText:(NSString *)test
{
    /* 編集不可 */
    _logTextView.editable = NO;
    
    if (_logTextView.text == nil || [_logTextView.text length] == 0)
    {
        _logTextView.text = test;
    }
    else
    {
        _logTextView.text = [test stringByAppendingString: _logTextView.text];
    }
    
    if (_logTextView.text.length > 50000)
    {
        /* 表示文字数を制限する */
        _logTextView.text = [_logTextView.text substringToIndex:50000];
    }
    [_logTextView setNeedsDisplay];
}

/* ログ用のDateFormatter取得 */
- (NSDateFormatter *)_logDateFormatter:(NSString *)dateFormat
{
    NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
    [formatter setDateFormat:dateFormat];
    [formatter setTimeZone:[NSTimeZone timeZoneWithAbbreviation:@"JST"]];
    [formatter setLocale:[[NSLocale alloc] initWithLocaleIdentifier:@"en_US_POSIX"]];
    return formatter;
}

/* ログファイルのパス取得 */
- (NSString *)_locationLogFilePath
{
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory,
                                                         NSUserDomainMask,
                                                         YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    
    NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
    [formatter setDateFormat:@"yyyyMMdd"];
    [formatter setTimeZone:[NSTimeZone timeZoneWithAbbreviation:@"JST"]];
    NSString *dateStr = [formatter stringFromDate:[NSDate date]];
    
    NSString *path = [documentsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"LocationSample_%@.txt", dateStr]];
    
    return path;
}

/* ログファイルを作成する */
- (void)_createLocationLogFile:(NSString *)filePath
{
    NSFileManager *manager = [NSFileManager defaultManager];
    if ([manager fileExistsAtPath:filePath] == YES)
    {
    }
    else
    {
        [manager createFileAtPath:filePath contents:nil attributes:nil];
    }
}

/* ログファイルを保存する */
- (void)_saveLogFile:(NSString *)logString
{
    NSString *filePath = [self _locationLogFilePath];
    if (filePath != nil)
    {
        [self _createLocationLogFile:filePath];

        NSFileHandle *fileHandle = [NSFileHandle fileHandleForWritingAtPath:filePath];
        [fileHandle seekToEndOfFile];
        
        NSData *data = [logString dataUsingEncoding:NSUTF8StringEncoding];
        [fileHandle writeData:data];
        [fileHandle closeFile];
    }
}

/* ログデータの作成 */
- (NSString *)_createLogData:(LocationInfo *)locationInfo
{
    NSMutableString *log = nil;
    if (locationInfo != nil)
    {
        /* バッテリ残量取得 */
        int deviceBatteryLevel = 0;
        float batteryLevel = [UIDevice currentDevice].batteryLevel;
        if (batteryLevel >= 0) {
            deviceBatteryLevel = batteryLevel * 100;
        }
        
        /* 出力ログの形成 */
        log = [NSMutableString string];
        NSDateFormatter *formatter = [self _logDateFormatter:@"yyyyMMdd HH:mm:ss"];
        NSDate *timeStamp = locationInfo.timeStamp;
        [log appendString:[formatter stringFromDate:timeStamp]];
        [log appendFormat:@",%f", locationInfo.latitude];
        [log appendFormat:@",%f", locationInfo.longitude];
        [log appendFormat:@",%f", locationInfo.horizontalAccuracy];
        [log appendFormat:@",%f", locationInfo.verticalAccuracy];
        [log appendFormat:@",%f", locationInfo.altitude];
        [log appendFormat:@",%f", locationInfo.speed];
        [log appendFormat:@",%f", locationInfo.bearing];
        [log appendFormat:@",%d", deviceBatteryLevel];
        [log appendString:@"\n"];
    }

    return log;
}

/* ステータスログデータの作成 */
- (NSString *)_createStatusLogData:(MeasuringType)measureStatus reason:(MeasuringTypeChangeReason)reason
{
    NSMutableString *log = [NSMutableString string];
    
    NSDateFormatter *formatter = [self _logDateFormatter:@"yyyyMMdd HH:mm:ss"];
    NSString *dateStr = [formatter stringFromDate:[NSDate date]];
    [log appendString:dateStr];
    [log appendString:@","];

    BOOL flag = YES;
    if (measureStatus == MeasuringType_Significant)
    {
        flag = NO;
    }
    
    [log appendString:@"測位方法変更通知: "];
    if (reason == MeasuringTypeChange_Battery)
    {
        if (flag == YES)
        {
            [log appendString:@"バッテリ残量復帰により標準測位に変更"];
        }
        else
        {
            [log appendString:@"バッテリ残量低下により基地局測位に変更"];
        }
    }
    [log appendString:@"\n"];

    
    return log;
}

/* 通信結果をコンソールログに出力 */
- (void)_outputConnectionResultToConsole:(NSString *)log response:(NSHTTPURLResponse *)response data:(NSDictionary *)data error:(NSError *)error
{
    NSMutableString *debugLog = [NSMutableString string];
    [debugLog appendString:log];
    
    if (error != nil)
    {
        [debugLog appendFormat:@",code: %ld", error.code];
        [debugLog appendFormat:@",description: %@", error.localizedDescription];
    }
    else if (response != nil)
    {
        [debugLog appendFormat:@",code:%ld", response.statusCode];
        [debugLog appendFormat:@",response:%@", [LMaUtility dictionaryToJson:data]];
    }
   NSLog(@"%@", debugLog);
}

/* 文字列アラート表示 */
- (void)_showTextAlert:(NSString *)title message:(NSString *)message
{
    NSString *okButtonTitle = @"OK";
    
    if (NSClassFromString(@"UIAlertController") != nil)
    {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:title
                                                                                 message:message
                                                                          preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:okButtonTitle
                                                            style:UIAlertActionStyleDefault
                                                          handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
    }
}

/* 取得した位置を履歴に格納 */
- (void)_stackLocation:(LocationInfo *)location
{
    if (_dataManager.permission == NO)
    {
        _locationLogs = nil;
        return;
    }

    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_BACKGROUND, 0), ^(void) {
        @synchronized(self) {
            if (self->_locationLogs == nil)
            {
                self->_locationLogs = [NSMutableArray array];
            }

            // ライブラリから取得した位置をメモリに格納
            [self->_locationLogs addObject:location];

            // 必須項目チェック
            if (![self->_dataManager isRequiredAppSendingParameters])
            {
                [self _setLogTextInUIThread:@"アプリ設定にて未入力のサーバ送信項目があります\n\n"];
                return;
            }

            // メモリ蓄積上限未満は位置履歴を送信しない
            if ((int)self->_locationLogs.count < self->_dataManager.logUploadCount)
            {
                return;
            }

            // 認証トークンなし、または、トークン有効期間を超えた場合、認証トークン取得
            NSDate *now = [NSDate date];
            if (self->_tokenInfo[kTokenId] != nil &&
                [now compare:self->_tokenInfo[kTokenExpireDate]] <= NSOrderedSame)
            {
                // 位置履歴送信
                [self _requestUploadLogs];
            }
            else
            {
                // 認証トークン取得
                [self _requestGetToken:^{
                    // 位置履歴送信
                    [self _requestUploadLogs];
                }];
            }
        }
    });
}

/* トークン取得通信 */
- (void)_requestGetToken:(void (^)(void))successHandler
{
    NSError *error = nil;
    NSHTTPURLResponse *response = nil;
    NSDictionary *data = nil;

    ServerConnection *server = [[ServerConnection alloc]init];
    [server requestGetToken:_dataManager.tokenUrl clientId:_dataManager.appTokenClientID secret:_dataManager.appTokenSecret response:&response data:&data error:&error];
    if ([server isResponseOK:response] == YES)
    {
        NSString *tokenId = [data objectForKey:@"access_token"];
        NSNumber *expiresIn = [data objectForKey:@"expires_in"];
        if (tokenId != nil && [expiresIn intValue] > 0)
        {
            [self->_tokenInfo setObject:tokenId forKey:kTokenId];
            // トークン有効期限（日時）を設定
            NSDate *expiresDate = [NSDate dateWithTimeIntervalSinceNow:expiresIn.intValue];
            [self->_tokenInfo setObject:expiresDate forKey:kTokenExpireDate];

            [self _setLogTextInUIThread:@"トークン取得通信に成功しました\n\n"];
            [self _outputConnectionResultToConsole:@"TokenServer request succeeded" response:response data:data error:nil];
            successHandler();
        }
    }
    else
    {
        [self _setLogTextInUIThread:@"トークン取得通信に失敗しました\n\n"];
        [self _outputConnectionResultToConsole:@"TokenServer request failed" response:response data:data error:error];
    }
}

/* 位置履歴サーバ送信 */
- (void)_requestUploadLogs
{
    NSArray *logs;

    // 送信対象範囲を特定
    if ((int)_locationLogs.count > kSendLogCountMax)
    {
        NSIndexSet *indexes = [NSIndexSet indexSetWithIndexesInRange:NSMakeRange(0, kSendLogCountMax)];
        NSArray *indexArray = [_locationLogs objectsAtIndexes:(indexes)];
        logs = [NSMutableArray arrayWithArray:indexArray];
    }
    else
    {
        logs = [NSMutableArray arrayWithArray:_locationLogs];
    }

    NSDictionary *logDictionary = [self _createlocationLogDictionary:logs];
    NSString *jsonString = [LMaUtility dictionaryToJson:logDictionary];

    NSLog(@"Location Server request json [%@]", jsonString);

    NSError *error = nil;
    NSHTTPURLResponse *response = nil;
    NSDictionary *data = nil;

    ServerConnection *server = [[ServerConnection alloc]init];
    [server requestUploadLogs:_dataManager.locationUrl tokenId:_tokenInfo[kTokenId] apikey:_dataManager.appApiKey json:jsonString response:&response data:&data error:&error];
    if ([server isResponseOK:response] == YES)
    {
        [self _setLogTextInUIThread:@"位置履歴サーバ送信に成功しました\n\n"];
        [self _outputConnectionResultToConsole:@"LocationServer send succeeded" response:response data:data error:nil];

        // 送信が成功した履歴をメモリから削除
        [_locationLogs removeObjectsInArray:logs];

        // 未送信の位置履歴がある場合は、残りを送信する
        if ((int)_locationLogs.count > 0)
        {
            [self _requestUploadLogs];
        }
    }
    else
    {
        [self _setLogTextInUIThread:@"位置履歴サーバ送信に失敗しました\n\n"];
        [self _outputConnectionResultToConsole:@"LocationServer send failed" response:response data:data error:error];

        // 400エラーの場合、送信に失敗した履歴をメモリから削除
        if (response.statusCode == 400)
        {
            [_locationLogs removeObjectsInArray:logs];
        }
    }
}

/* 位置蓄積サーバ送信用Dictionary生成 */
- (NSDictionary *)_createlocationLogDictionary:(NSArray *)logs
{
    if(!logs.count) {
        return nil;
    }

    NSMutableDictionary *dic = [NSMutableDictionary dictionary];
    // deviceIDをセット
    [dic setValue:_dataManager.deviceID forKey:@"deviceId"];
    NSMutableArray *positions = [[NSMutableArray alloc] init];

    for (LocationInfo *info in logs) {
        NSMutableDictionary *pos= [[NSMutableDictionary alloc] init];

        /* 検索DB可能フラグ */
        [pos setValue:[NSNumber numberWithBool:_dataManager.isSearch] forKey:@"isSearch"];
        /* 位置取得日時 */
        NSDate *date = info.timeStamp;
        NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
        [dateFormatter setDateFormat: @"yyyy'-'MM'-'dd'T'HH':'mm':'ss"];
        NSString *timeString = [dateFormatter stringFromDate:date];
        [pos setValue:timeString forKey:@"time"];
        /* 緯度 */
        [pos setValue:[NSNumber numberWithDouble:info.latitude] forKey:@"lat"];
        /* 経度 */
        [pos setValue:[NSNumber numberWithDouble:info.longitude] forKey:@"lon"];
        /* 水平精度 */
        if (info.horizontalAccuracy >= 0) {
            [pos setValue:[NSNumber numberWithDouble:info.horizontalAccuracy] forKey:@"accuracy"];
        }
        /* 標高 */
        if (info.verticalAccuracy >= 0) {
            [pos setValue:[NSNumber numberWithDouble:info.altitude] forKey:@"altitude"];
        }
        /* 速度 */
        if (info.speed >= 0) {
            [pos setValue:[NSNumber numberWithDouble:info.speed] forKey:@"speed"];
        }
        /* 方位 */
        if (info.bearing >= 0) {
            [pos setValue:[NSNumber numberWithDouble:info.bearing] forKey:@"bearing"];
        }
        /* 付加情報 */
        NSString *extra = _dataManager.extra;
        if(extra != nil && extra.length > 0) {
            [pos setValue:extra forKey:@"extra"];
        }
        /* 非検索付加情報 */
        NSString *unsearchableExtra = _dataManager.unsearchableExtra;
        if(unsearchableExtra != nil && unsearchableExtra.length > 0) {
            [pos setValue:unsearchableExtra forKey:@"unsearchableExtra"];
        }
        /* １回分の測位のログのDictionaryを配列に追加 */
        [positions addObject:pos];
    }
    // positionsをセット
    [dic setValue:positions forKey:@"positions"];

    return  dic;
}

/* ログ画面に出力 */
- (void)_setLogTextInUIThread:(NSString *)log
{
    dispatch_async(dispatch_get_main_queue(), ^{
        [self _setLogText:log];
    });
}

@end
